""" The core file for the Query package, which connects to the database

:Author: Bilal Shaikh < bilalshaikh42@gmail.com >
:Date: 2019-11-07
:Copyright: 2019, Karr Lab
:License: MIT
"""
from biosimulations_query import config
from biosimulations_query.users import UserManager
from biosimulations_query.resources import SimulationManager
from biosimulations_query.resources import ModelManager
from biosimulations_query.resources import ProjectManager
from biosimulations_query.resources import ChartManager
from biosimulations_query.resources import VisualizationManager
from biosimulations_query.files import FileManager
from biosimulations_query.files.model_file_manager import ModelFileManager
from biosimulations_query.files.image_file_manger import ImageFileManager


class DBManager:
    def __init__(
            self,
            database=None,
            username=None,
            password=None,
            server=None,
            authSource="admin"):
        self.username = username
        self.password = password
        self.server = server
        self.database = database
        self.authSource = authSource
        if(not self.username):
            self.username = config.Config.USERNAME
        if (not self.password):
            self.password = config.Config.PASSWORD
        if (not self.server):
            self.server = config.Config.SERVER
        if(not database):
            self.database = config.Config.DATABASE
        # self.authDB = config.Config.AUTHDB
        # self.read_preference = config.Config.READ_PREFERENCE

    def user_manager(self, collection="users"):
        return UserManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            collection=collection)

    def simulation_manager(self, collection="simulations"):
        return SimulationManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            authSource=self.authSource,
            collection=collection,
            user_manager=self.user_manager())

    def model_manager(self, collection="models"):
        return ModelManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            authSource=self.authSource,
            collection=collection,
            user_manager=self.user_manager())

    def project_manager(self, collection="projects"):
        return ProjectManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            authSource=self.authSource,
            collection=collection,
            user_manager=self.user_manager())

    def chart_manager(self, collection="charts"):
        return ChartManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            authSource=self.authSource,
            collection=collection,
            user_manager=self.user_manager())

    def visualization_manager(self, collection="visualizations"):
        return VisualizationManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            authSource=self.authSource,
            collection=collection,
            user_manager=self.user_manager())

    def file_manager(self, collection="fs"):
        return FileManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            collection=collection,
            authSource=self.authSource,
            userManager=self.user_manager()
        )

    def model_file_manager(self, collection="fs"):
        return ModelFileManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            collection=collection,
            authSource=self.authSource,
            userManager=self.user_manager()
        )

    def image_file_manager(self, collection="fs"):
        return ImageFileManager(
            username=self.username,
            password=self.password,
            host=self.server,
            database=self.database,
            authSource=self.authSource,
            collection=collection,
            userManager=self.user_manager()
        )


if __name__ == "__main__":
    dbm = DBManager()
    um = dbm.user_manager()
