""" file_manager

:Author: Bilal Shaikh < bilalshaikh42@gmail.com >
:Date: 2020-02-25
:Copyright: 2020, Karr Lab
:License: MIT
"""
from biosimulations_query.utils import MongoUtil
import gridfs
import bson
from biosimulations_query.utils import Pipelines


class FileManager(object):

    def __init__(
            self,
            username,
            password,
            host,
            authSource,
            database,
            collection,
            userManager):

        self.mongo_manager = MongoUtil(
            host=host,
            username=username,
            password=password,
            authSource=authSource,
            db=database,
        )
        self.collection_name = collection
        self.client, self.db, self.collection = self.mongo_manager.con_db(
            collection)
        self.fs = gridfs.GridFS(self.db, collection=collection)

        self.userManager = userManager

    def add_file_to_db(self, file, file_name, file_type, owner, private=False, metadata={}):
        owner = self.userManager.get_id_from_username(owner)
        object_id = self.fs.put(
            file,
            filename=file_name,
            owner=owner,
            file_type=file_type,
            private=False,
            metadata=metadata)
        object_id = str(object_id)
        return object_id

    def get_file_from_db_by_id(self, file_id):
        file_id = bson.ObjectId(file_id)
        return self.fs.get(file_id)

    def get_file_metadata_from_db_by_id(self, file_id):
        file_id = bson.ObjectId(file_id)
        file_data = self.db[self.collection_name].files.aggregate(Pipelines.match_file_id(file_id) +
                                                                  Pipelines.convert_object_id_to_id()+Pipelines.convert_owner_id_to_username())
        return list(file_data)[0]

    def get_all_files_metadata_from_db(self):
        file_data = self.db[self.collection_name].files.aggregate(
            Pipelines.convert_object_id_to_id()+Pipelines.convert_owner_id_to_username())
        print(file_data)

        return list(file_data)

    def get_all_public_files_metadata_from_db(self):
        pipeline = Pipelines.get_public()+Pipelines.convert_object_id_to_id() + \
            Pipelines.convert_owner_id_to_username()
        file_data = self.db[self.collection_name].files.aggregate(pipeline)

        return list(file_data)

    def get_private_files_metadata_from_db_by_owner(self, username):
        owner = self.userManager.get_id_from_username(username)
        pipeline = Pipelines.get_private_by_owner(
            owner)+Pipelines.convert_object_id_to_id() + Pipelines.convert_owner_id_to_username()
        file_data = self.db[self.collection_name].files.aggregate(pipeline)

    def replace_file_contents(self, file_id, file, filename=None, owner=None):
        file_id = bson.ObjectId(file_id)
        old_file = self.fs.get(file_id)
        if(filename is None):
            filename = old_file.filename
        if(owner) is None:
            owner = old_file.owner
        else:
            owner = self.userManager.get_id_from_username(owner)
        file_metadata = old_file.metadata

        self.fs.delete(file_id)
        new_file_id = self.fs.put(
            file,
            filename=filename,
            owner=owner,
            metadata=file_metadata)
        return str(new_file_id)

    def replace_file_data(
            self,
            file_id,
            filename,
            file_type,
            owner,
            private=False,
            user_collection="users",
            metadata={}):
        file_id = bson.ObjectId(file_id)
        owner = self.userManager.get_id_from_username(owner)
        self.db[self.collection_name].files.update_one({'_id': file_id}, {'$set': {
            'owner': owner, 'filename': filename, 'file_type': file_type, 'private': private, 'metadata': metadata}})
        pipeline = Pipelines.match_file_id(
            file_id) + Pipelines.convert_object_id_to_id()+Pipelines.convert_owner_id_to_username(user_collection)
        file_data = self.db[self.collection_name].files.aggregate(
            pipeline)
        file_list = list(file_data)
        return file_list[0]
