class Error(Exception):
    """Base class for exceptions in this module."""
    pass


class DoesNotExistError(Error):
    """ Base class for exceptions in which a resource is not found.

    Attributes:
        identifier: The id of the resource
        resource: The name of the resource
    """

    def __init__(self, identifier, resource="resource"):
        self.identifier = identifier
        self.resource = resource

    def __str__(self):
        return ("The {0} with id {1} does not exist").format(
            self.resource, self.identifier)


class UserDoesNotExistError(DoesNotExistError):
    def __init__(self, identifier, resource='user'):
        super().__init__(identifier, resource=resource)

    def __str__(self):
        return ("The {0} with username {1} does not exist").format(
            str(self.resource), str(self.identifier))


class IdExistsError(Error):
    def __init__(self, identifier=None):
        self.identifier = identifier

    def __str__(self):
        return ("The ID {0} already exists").format(
            ',' + str(self.identifier) + ',')
